#!/usr/bin/env python
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Data: 2014.07.31
# Autor: Luis Cardoso

import sys
from gettext import gettext as _
from gi.repository import Gtk, Gdk, GdkPixbuf, WebKit, GLib

GLib.threads_init()
DEFAULT_PAGE = "ubuntued.info"
HOME_PAGE = "duckduckgo.com"
APP_NAME = "Simple Web Browser"
NEW_TAB_DEFAULT = "New tab"

class SWBMainToolbar(Gtk.Toolbar):
    def __init__(self, *args, **kwargs):
        super(SWBMainToolbar, self).__init__(*args, **kwargs)
        
        # Define as Primary Toolbar
        self.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        
        # Define as Back Button
        self.go_back = Gtk.ToolButton(Gtk.STOCK_GO_BACK)
        self.go_back.set_sensitive(False)
        self.go_back.set_visible(False)
        self.insert(self.go_back, -1)
        
        # Define as Forward Button
        self.go_forward = Gtk.ToolButton(Gtk.STOCK_GO_FORWARD)
        self.go_forward.set_sensitive(False)
        self.go_forward.set_visible(False)
        self.insert(self.go_forward, -1)
        
        # Define as Home Button
        self.go_home = Gtk.ToolButton(Gtk.STOCK_HOME)
        self.go_home.set_sensitive(True)
        self.go_home.set_visible(True)
        self.insert(self.go_home, -1)
        
        # Define as URL Entry
        self.url_entry = Gtk.Entry()
        self.url_entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY, "applications-internet")
        self.url_entry.set_icon_sensitive(Gtk.EntryIconPosition.PRIMARY, True)
        self.url_entry.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_REFRESH)
        self.url_entry.set_icon_sensitive(Gtk.EntryIconPosition.SECONDARY, True)
        self.url_entry.set_placeholder_text="url"
        self.url_entry.set_text(DEFAULT_PAGE)
        self.url_item = Gtk.ToolItem()
        self.url_item.set_expand(True)
        self.url_item.add(self.url_entry)
        self.insert(self.url_item, -1)
        
        # Define as Properties Button
        self.go_properties = Gtk.ToolButton(Gtk.STOCK_PROPERTIES)
        self.go_properties.set_sensitive(True)
        self.go_properties.set_visible(True)
        self.insert(self.go_properties, -1)

class SWBFindToolbar(Gtk.Toolbar):
    def __init__(self, *args, **kwargs):
        super(SWBFindToolbar, self).__init__(*args, **kwargs)
        
        # Define Find Close Button
        close_find = Gtk.ToolButton(Gtk.STOCK_CLOSE)
        close_find.set_sensitive(True)
        close_find.set_visible(True)
        self.insert(close_find, -1)
        close_find.connect("clicked", lambda x: self.hide())
        
        # Define Find Tool Item
        find_item = Gtk.ToolItem()
        find_item.set_expand(True)
        self.insert(find_item, -1)
        
        # Define Find Search Entry
        self.find_entry = Gtk.Entry()
        self.find_entry.set_icon_from_stock(Gtk.EntryIconPosition.PRIMARY, Gtk.STOCK_FIND)
        self.find_entry.set_icon_sensitive(Gtk.EntryIconPosition.PRIMARY, False)
        self.find_entry.set_placeholder_text="search"
        find_item.add(self.find_entry)
        
        # Define Find Previous
        self.find_previous = Gtk.ToolButton(Gtk.STOCK_GO_UP)
        self.find_previous.set_sensitive(True)
        self.find_previous.set_visible(True)
        self.insert(self.find_previous, -1)
        
        # Define Find Previous
        self.find_next = Gtk.ToolButton(Gtk.STOCK_GO_DOWN)
        self.find_next.set_sensitive(True)
        self.find_next.set_visible(True)
        self.insert(self.find_next, -1)

class SWBTab(Gtk.VBox):
    def __init__(self, *args, **kwargs):
        super(SWBTab, self).__init__(*args, **kwargs)
        
        # Toolbar
        self.main_toolbar = SWBMainToolbar()
        
        # WebView
        self.webview = WebKit.WebView()
        self.is_loading = 0
        self._load_url()
#        self.webview.connect("title-changed", self.titleChanged)
        self.webview.connect("load-progress-changed", self._update_entry)
        self.webview.connect("load-started", self._on_load_started)
        self.webview.connect("load-finished", self._on_load_finished)
        self.webview.connect("icon-loaded", self._on_load_finished)
        scrolled_window = Gtk.ScrolledWindow()
        scrolled_window.add(self.webview)
        
        # Find Toolbar
        self.find_toolbar = SWBFindToolbar()
        
        # Merge all together
        self.pack_start(self.main_toolbar, False, True, 0)
        self.pack_start(scrolled_window, True, True, 0)
        self.pack_start(self.find_toolbar, False, True, 0)
        
        # Main Toolbar
        # Go Back
        self.main_toolbar.go_back.connect("clicked", lambda x: self.webview.go_back())
        # Go Forward
        self.main_toolbar.go_forward.connect("clicked", lambda x: self.webview.go_forward())
        # Go Home
        self.main_toolbar.go_home.connect("clicked", lambda x: self._load_about_pages("home"))
        # Entry
        self.main_toolbar.url_entry.connect("activate", self._load_url)
        self.main_toolbar.url_entry.connect("icon-press", self._on_icon_pressed)
        # Go URL
        #self.main_toolbar.go_properties.connect("clicked", self.clickTbEvent, "go-url")
        
        # Find Toolbar
        # Find Entry
        self.find_toolbar.find_entry.connect("activate",
            lambda x: self.webview.search_text(self.find_toolbar.find_entry.get_text(),
            False, True, True))
        
        self.find_toolbar.find_previous.connect("clicked",
            lambda x: self.webview.search_text(self.find_toolbar.find_entry.get_text(),
            False, False, True))
        
        self.find_toolbar.find_next.connect("clicked",
            lambda x: self.webview.search_text(self.find_toolbar.find_entry.get_text(),
            False, True, True))
        
        self.main_toolbar.show_all()
        scrolled_window.show_all()
        self.show()
    
    def _on_load_started(self, widget, frame):
        self.main_toolbar.url_entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY, "applications-internet")
        self.main_toolbar.url_entry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "image-loading")
        self.main_toolbar.url_entry.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_CLOSE)
        self.is_loading = 1

    def _on_load_finished(self, widget, frame):
        self.main_toolbar.url_entry.set_icon_from_stock(Gtk.EntryIconPosition.SECONDARY, Gtk.STOCK_REFRESH)
        icon = self.webview.get_icon_pixbuf()
        if icon != None:
            #print icon.get_width(), 'x', icon.get_height()
            self.main_toolbar.url_entry.set_icon_from_pixbuf(Gtk.EntryIconPosition.PRIMARY, icon.scale_simple(16,16, GdkPixbuf.InterpType.BILINEAR))
        else:
            self.main_toolbar.url_entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY, "applications-internet")
    
    def _on_icon_pressed(self, entry, icon_pos, event):
        if icon_pos == Gtk.EntryIconPosition.PRIMARY:
            # Pop Up Option
            self.webview.reload()
        elif icon_pos == Gtk.EntryIconPosition.SECONDARY:
            if self.is_loading:
                self.webview.stop_loading()
            else:
                self.webview.reload()
    
    def _load_url(self, widget=None):
        url = self.main_toolbar.url_entry.get_text()
        #self.window.set_title("My Simple Web Browser")
        if "about:" in url:
            self._load_about_pages(url.split(":")[1])
        elif not url:
            url = "about:blank"
            self._load_about_pages("blank")
        elif not "://" in url:
            url = "http://" + url
            self.webview.load_uri(url)
        else:
            self.webview.load_uri(url)
    
    def _load_about_pages(self, name):
        baseUrl = "about:"+name
        if name == "home":
            self.main_toolbar.url_entry.set_text(HOME_PAGE)
            self._load_url()
        else:
            self.webview.load_html_string("<h1>"+baseUrl+"</h1>", "")
    
    def _update_entry(self, widget=None, data=None):
        urlEntry = self.main_toolbar.url_entry.get_text()
        urlWeb = self.webview.props.uri
        if not urlWeb:
            self.main_toolbar.url_entry.set_text("")
        elif (urlEntry != urlWeb):
            self.main_toolbar.url_entry.set_text(self.webview.props.uri)
        self.main_toolbar.go_back.set_sensitive(self.webview.can_go_back())
        self.main_toolbar.go_back.set_visible(self.webview.can_go_back())
        self.main_toolbar.go_forward.set_sensitive(self.webview.can_go_forward())
        self.main_toolbar.go_forward.set_visible(self.webview.can_go_forward())

class SWBWindow(Gtk.Window):
    def __init__(self, *args, **kwargs):
        super(SWBWindow, self).__init__(*args, **kwargs)
        
        self.set_title(APP_NAME)
        self.set_icon_name("applications-internet")
        self.set_resizable(True)
        self.resize(800,600)
        self.set_decorated(True)
        
        # create notebook and tabs
        self.notebook = Gtk.Notebook()
        self.notebook.set_scrollable(True)

        # basic stuff
        self.tabs = []
        
        # create a first, empty browser tab
        #self.tabs.append((self._create_tab(), Gtk.Label("New Tab")))
        #self.notebook.append_page(*self.tabs[0])
        self._hovered_uri = None
        self._open_new_tab()
        self.add(self.notebook)
        
        # connect signals
        self.connect("destroy", Gtk.main_quit)
        self.connect("key-press-event", self._key_pressed)
        self.notebook.connect("switch-page", self._tab_changed)

        self.notebook.show()
        self.show()
    
    def _hovering_over_link(self, view, title, uri):
        self._hovered_uri = uri
        #self.emit("hover-changed", view, uri)
        #print self._hovered_uri
    
    def _populate_page_popup(self, view, menu):
        if self._hovered_uri:
            open_in_new_tab = Gtk.MenuItem(_("Abrir link em nova aba"))
            open_in_new_tab.connect("activate", self._open_in_new_tab, view)
            menu.insert(open_in_new_tab, 0)
            menu.show_all()
#            for x in menu:
#                print x.get_label(),x.get_events()
            #print "popup:",self._hovered_uri
    
    def _open_in_new_tab(self, menuitem, view):
        self._open_new_tab(self._hovered_uri)
        #print "open_in_new_tab",self._hovered_uri
        
    def _tab_changed(self, notebook, current_page, index):
        if (index is None):
            index = 0
        title = self.tabs[index][0].webview.get_title()
        if title is not None:
            self.set_title(title+" - "+APP_NAME)
        else:
            self.set_title(APP_NAME)

    def _title_changed(self, webview, frame, title):
        current_page = self.notebook.get_current_page()

        counter = 0
        for tab, label in self.tabs:
            if tab.webview is webview:
                if len(title) < 15:
                    label.get_children()[0].set_text(title)
                else:
                    label.get_children()[0].set_text(title[:15]+"...")
                label.set_tooltip_markup("<b>Title:</b> "+title)
#                print label.get_parent(),self.notebook.get_focus_child()
                #print "counter:",counter,"current page:",current_page
                if counter == current_page:
                    self._tab_changed(None, None, counter)
                    #print counter
                break
            counter += 1

    def _create_tab(self, url=None):
        tab = SWBTab()
        tab.webview.connect("title-changed", self._title_changed)
        tab.webview.connect("new-window-policy-decision-requested", self._new_tab_requested)
        tab.webview.connect("hovering-over-link", self._hovering_over_link)
        tab.webview.connect("populate-popup", self._populate_page_popup)
        if url is not None:
            tab.webview.load_uri(url)
        return tab

    def _reload_tab(self):
        self.tabs[self.notebook.get_current_page()][0].webview.reload()

    def _close_current_tab(self):
        if self.notebook.get_n_pages() == 1:
            return
        page = self.notebook.get_current_page()
        current_tab = self.tabs.pop(page)
        self.notebook.remove(current_tab[0])

    def _open_new_tab(self, url=None):
        current_page = self.notebook.get_current_page()
        #print current_page
        tablabel = Gtk.HBox()
        self.label = Gtk.Label("New Tab")
        close_icon = Gtk.Image()
        close_icon.set_from_stock(Gtk.STOCK_CLOSE,Gtk.IconSize.MENU)
        close = Gtk.Button()
        close.set_image(close_icon)
        close.set_relief(Gtk.ReliefStyle.NONE)
        tablabel.pack_start(self.label,True,True,0)
        tablabel.pack_start(close,False,False,1)
        #tablabel.set_data("label",self.label)
        tablabel.show_all()
        
        #page_tuple = (self._create_tab(url), Gtk.Label("New Tab"))
        page_tuple = (self._create_tab(url), tablabel)
        self.tabs.insert(current_page+1, page_tuple)
        self.notebook.insert_page(page_tuple[0], page_tuple[1], current_page+1)
        self.notebook.set_current_page(current_page+1)
    
    def _new_tab_requested(self, view, frame=None, request=None, nav_action=None, policy_decision=None):
        self._open_new_tab(request.get_uri())
        policy_decision.ignore()
        return True

    def _focus_url_bar(self):
        current_page = self.notebook.get_current_page()
        self.tabs[current_page][0].main_toolbar.url_entry.grab_focus()

    def _raise_find_dialog(self):
        current_page = self.notebook.get_current_page()
        self.tabs[current_page][0].find_toolbar.show_all()
        self.tabs[current_page][0].find_toolbar.find_entry.grab_focus()
    

    def _key_pressed(self, widget, event):
        modifiers = Gtk.accelerator_get_default_mod_mask()
        mapping = {Gdk.KEY_r: self._reload_tab,
                   Gdk.KEY_w: self._close_current_tab,
                   Gdk.KEY_t: self._open_new_tab,
                   Gdk.KEY_l: self._focus_url_bar,
                   Gdk.KEY_f: self._raise_find_dialog,
                   Gdk.KEY_q: Gtk.main_quit}

        if event.state & modifiers == Gdk.ModifierType.CONTROL_MASK \
          and event.keyval in mapping:
            mapping[event.keyval]()
        
if __name__ == "__main__":
    Gtk.init(sys.argv)
    browser = SWBWindow()
    Gtk.main()

